/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Xul Explorer.
 *
 * The Initial Developer of the Original Code is Mark Finkle.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Finkle, <mark.finkle@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// ------------------------------------------------------------------
// use helper functions to hook up the explorer object so "this"
// works in the explorer object
// ------------------------------------------------------------------

function explorer_startup() {
  explorer.startup();
}

function explorer_shutdown() {
  explorer.shutdown();
}


// ------------------------------------------------------------------
// attach to window events so explorer object can startup / shutdown
// ------------------------------------------------------------------

window.addEventListener("load", explorer_startup, false);
window.addEventListener("unload", explorer_shutdown, false);


// ------------------------------------------------------------------
// explorer object
// ------------------------------------------------------------------

var explorer = {
  initialized : false,
  editor : null,
  previewInline : null,
  previewWindow : null,
  prefService : null,
  
  _initSnippets : function() {
    // init the snippets
    snippetManager.init(this, "chrome://explorer/content/snippets.json", this.prefService.getCharPref("explorer.snippets.user"));

    var stringbundle = document.getElementById("strings_explorer");
    var allCategory = stringbundle.getString("categoryAll.category");
    
    var categoryList = document.getElementById("snippet-type");
    categoryList.appendItem(allCategory, -1);

    var categories = snippetManager.categories;
    for each(var category in categories) {
      categoryList.appendItem(category, categories.indexOf(category));
    }

    // let's get the startup snippet category in the menulist
    var startupCategory = this.prefService.getCharPref("explorer.startup.category");
    var startupIndex = categories.indexOf(startupCategory);
    categoryList.selectedIndex = startupIndex + 1;

    // fill the list based on the default category
    var snippetList = document.getElementById("snippets-list");
    var snippetIndices = snippetManager.filterByCategory(startupIndex);
    for (var iSnippet in snippetIndices) {
      var templateIndex = snippetIndices[iSnippet];
      snippetList.appendItem(snippetManager.snippets[templateIndex].title, templateIndex);
    }

    // Add the special 'templates' snippets to the file | new submennu
    var fileNewCategory = stringbundle.getString("fileNew.category");

    var categoryIndex = snippetManager.categories.indexOf(fileNewCategory);
    var templateIndices = snippetManager.filterByCategory(categoryIndex);
    var newPopup = document.getElementById("popup_new");
    for (var iIndex in templateIndices) {
      var templateIndex = templateIndices[iIndex];
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("label", snippetManager.snippets[templateIndex].title);
      menuitem.setAttribute("class", "menuitem-iconic menu-new-template");
      menuitem.setAttribute("oncommand", "doLoadSnippet(" + templateIndex + ");");
      
      newPopup.appendChild(menuitem);
    }
  },
    
  _initEditor : function() {
    this.editor.makeEditable("html", false);
  
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.enableUndo(true);
    textEditor.rootElement.style.fontFamily = "-moz-fixed";
    textEditor.rootElement.style.fontSize = "10pt";
    textEditor.rootElement.style.whiteSpace = "nowrap";
    textEditor.rootElement.style.margin = 0;

    // let's get the startup snippet script to load into the editor
    var stringbundle = document.getElementById("strings_explorer");
    var scriptCategory = stringbundle.getString("fileNew.category");
    var startupScript = this.prefService.getCharPref("explorer.startup.script");

    var snippets = snippetManager.snippets;
    for (var iSnippet in snippets) {
      if (snippets[iSnippet].category == scriptCategory && snippets[iSnippet].title == startupScript)
        textEditor.insertText(snippets[iSnippet].code);
    }
  },
  
  startup : function() {
    if (this.initialized)
      return;
    this.initialized = true;

    this.prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                 .getService(Components.interfaces.nsIPrefBranch2);
  
    // locate and cache elements
    this.editor = document.getElementById("source");
    this.previewInline = document.getElementById("preview");
  
    // initialize
    this._initSnippets();
    this._initEditor();

    XULChecker.init("chrome://explorer/content/xulschema.json");

    livePreview.init(this);

    // hook up an observer on the auto preview pref
    this.prefService.addObserver(autoRefreshObserver.pref, autoRefreshObserver, false);
    autoRefreshObserver.update();

    consoleErrors.startup(this);

    fileController.init(this);
    viewController.init(this);
    toolsController.init(this);
    helpController.init(this);
    
    window.controllers.appendController(fileController);
    window.controllers.appendController(viewController);
    window.controllers.appendController(toolsController);
    window.controllers.appendController(helpController);
  },
  
  shutdown : function() {
    consoleErrors.shutdown();
  },

  clearSource : function() {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.enableUndo(false);
    textEditor.selectAll();
    textEditor.deleteSelection(textEditor.eNone);
    textEditor.enableUndo(true);
    textEditor.resetModificationCount();
    
    this.editor.contentWindow.focus();
    CommandUpdater.doCommand("cmd_moveTop");
  },
    
  getSource : function() {
    var source = this.editor.getEditor(this.editor.contentWindow).outputToString("text/plain", 2);
    return source;
  },
  
  setSource : function(aText) {
    this.clearSource();
    this.insertText(aText);
  },

  insertText : function(aText) {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.insertText(aText);
  },

  refreshPreview : function() {
    this.hasErrors(false);
    
    var source = this.getSource();
    var dataURI = "data:application/vnd.mozilla.xul+xml," + encodeURIComponent(source);
    
    this.previewInline.setAttribute("src", dataURI);
    
    if (this.previewWindow != null && this.previewWindow.closed == false) {
      this.previewWindow.document.location = dataURI;
    }
  },
  
  openPreview : function() {
    if (this.previewWindow == null || this.previewWindow.closed == true) {
      this.previewWindow = window.open("about:blank", "explorer-preview", "chrome,dependent,all,centerscreen,resizable=yes,width=400,height=400");
      setTimeout(this.refreshPreview(), 100);
    }
    return this.previewWindow;
  },
  
  getHelpURI : function(cmd) {
    var stringbundle = document.getElementById("strings_explorer");
    var helpURI;
    switch (cmd) {
      case "cmd_help": 
        helpURI = stringbundle.getString("appReference.url");
        break;
      case "cmd_xulref": 
        helpURI = stringbundle.getString("xulReference.url");
        break;
      case "cmd_xultut": 
        helpURI = stringbundle.getString("xulTutorial.url");
        break;
      case "cmd_jsref": 
        helpURI = stringbundle.getString("jsReference.url");
        break;
      case "cmd_keyword": 
      {
        var keyword = this.editor.contentWindow.getSelection().toString();
        if (keyword.length > 0)
          helpURI = stringbundle.getString("xulKeyword.url") + keyword;
        else
          helpURI = stringbundle.getString("xulReference.url");
        break;
      }
    }
    return helpURI;
  },
  
  getEditor : function() {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    return textEditor;
  },
  
  hasErrors : function(errors) {
    var errorImg = document.getElementById("img_jsconsole");
    if (errors) {
      errorImg.setAttribute("mode", "error");
    }
    else {
      errorImg.setAttribute("mode", "ok");
    }
  }
};


// ------------------------------------------------------------------
// helper object that watches for autorefresh pref changes
// ------------------------------------------------------------------
var autoRefreshObserver = {
  pref : "explorer.preview.autorefresh",
  
  observe: function (aSubject, aTopic, aPrefName)
  {
    if (aTopic != "nsPref:changed" || aPrefName != this.pref)
      return;

    this.update();
  },

  update: function ()
  {
    livePreview.enabled = explorer.prefService.getBoolPref(this.pref);
  }
};


// ------------------------------------------------------------------
// helper methods to handle snippets
// TODO: move to an object
// ------------------------------------------------------------------

function doInsertSnippet() {
  var snippetList = document.getElementById("snippets-list");
  if (snippetList.selectedItem) {
    snippetManager.insert(snippetList.selectedItem.value);  
  }
}

// need to figure out sending params with commands so I can move this to fileController
function doLoadSnippet(templateIndex) {
  explorer.clearSource();
  snippetManager.insert(templateIndex);  
}

function doSnippetCateorgyChange() {
    var categoryList = document.getElementById("snippet-type");
    if (categoryList.selectedItem) {
      // save the current category
      explorer.prefService.setCharPref("explorer.startup.category", categoryList.selectedItem.label);

      // clear the list of snippets
      var snippetList = document.getElementById("snippets-list");
      while (snippetList.getRowCount() > 0) {
        snippetList.removeItemAt(0);
      }

      // get a list of snippets in the given category and fill the list        
      var snippetIndices = snippetManager.filterByCategory(categoryList.selectedItem.value);
      for (var iSnippet in snippetIndices) {
        var templateIndex = snippetIndices[iSnippet];
        snippetList.appendItem(snippetManager.snippets[templateIndex].title, templateIndex);
      }
    }
}
